@@ -22,7 +22,6 @@ __all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
     'TaxRuleLineTemplate', 'TaxRuleLine',
     'OpenTaxCode',
     'AccountTemplateTaxTemplate', 'AccountTemplate2', 'AccountTax', 'Account2']
-__metaclass__ = PoolMeta
 
 KINDS = [
     ('sale', 'Sale'),
@@ -42,8 +41,7 @@ class TaxGroup(ModelSQL, ModelView):
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
         super(TaxGroup, cls).__register__(module_name)
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
 
         # Migration from 1.4 drop code_uniq constraint
         table.drop_constraint('code_uniq')
@@ -90,14 +88,13 @@ class TaxCodeTemplate(ModelSQL, ModelView):
             res['template'] = self.id
         return res
 
-    def create_tax_code(self, company_id, template2tax_code=None,
-            parent_id=None):
+    @classmethod
+    def create_tax_code(cls, account_id, company_id, template2tax_code=None):
         '''
         Create recursively tax codes based on template.
         template2tax_code is a dictionary with tax code template id as key and
         tax code id as value, used to convert template id into tax code. The
         dictionary is filled with new tax codes.
-        Return the id of the tax code created
         '''
         pool = Pool()
         TaxCode = pool.get('account.tax.code')
@@ -105,24 +102,31 @@ class TaxCodeTemplate(ModelSQL, ModelView):
         if template2tax_code is None:
             template2tax_code = {}
 
-        if self.id not in template2tax_code:
-            vals = self._get_tax_code_value()
-            vals['company'] = company_id
-            vals['parent'] = parent_id
+        def create(templates):
+            values = []
+            created = []
+            for template in templates:
+                if template.id not in template2tax_code:
+                    vals = template._get_tax_code_value()
+                    vals['company'] = company_id
+                    if template.parent:
+                        vals['parent'] = template2tax_code[template.parent.id]
+                    else:
+                        vals['parent'] = None
+                    values.append(vals)
+                    created.append(template)
 
-            new_tax_code, = TaxCode.create([vals])
+            tax_codes = TaxCode.create(values)
+            for template, tax_code in zip(created, tax_codes):
+                template2tax_code[template.id] = tax_code.id
 
-            prev_data = {}
-            for field_name, field in self._fields.iteritems():
-                prev_data[field_name] = getattr(self, field_name)
-            template2tax_code[self.id] = new_tax_code.id
-        new_id = template2tax_code[self.id]
-
-        new_childs = []
-        for child in self.childs:
-            new_childs.append(child.create_tax_code(company_id,
-                    template2tax_code=template2tax_code, parent_id=new_id))
-        return new_id
+        childs = cls.search([
+                ('account', '=', account_id),
+                ('parent', '=', None),
+                ])
+        while childs:
+            create(childs)
+            childs = sum((c.childs for c in childs), ())
 
 
 class TaxCode(ModelSQL, ModelView):
@@ -171,7 +175,7 @@ class TaxCode(ModelSQL, ModelView):
 
     @classmethod
     def get_sum(cls, codes, name):
-        cursor = Transaction().cursor
+        cursor = Transaction().connection.cursor()
         res = {}
         pool = Pool()
         MoveLine = pool.get('account.move.line')
@@ -227,6 +231,15 @@ class TaxCode(ModelSQL, ModelView):
             (cls._rec_name,) + tuple(clause[1:]),
             ]
 
+    @classmethod
+    def copy(cls, codes, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('template')
+        return super(TaxCode, cls).copy(codes, default=default)
+
     @classmethod
     def delete(cls, codes):
         codes = cls.search([
@@ -234,25 +247,33 @@ class TaxCode(ModelSQL, ModelView):
                 ])
         super(TaxCode, cls).delete(codes)
 
-    def update_tax_code(self, template2tax_code=None):
+    @classmethod
+    def update_tax_code(cls, company_id, template2tax_code=None):
         '''
         Update recursively tax code based on template.
         template2tax_code is a dictionary with tax code template id as key and
         tax code id as value, used to convert template id into tax code. The
         dictionary is filled with new tax codes
         '''
-
         if template2tax_code is None:
             template2tax_code = {}
 
-        if self.template:
-            vals = self.template._get_tax_code_value(code=self)
-            if vals:
-                self.write([self], vals)
-            template2tax_code[self.template.id] = self.id
-
-        for child in self.childs:
-            child.update_tax_code(template2tax_code=template2tax_code)
+        values = []
+        childs = cls.search([
+                ('company', '=', company_id),
+                ('parent', '=', None),
+                ])
+        while childs:
+            for child in childs:
+                if child.template:
+                    vals = child.template._get_tax_code_value(code=child)
+                    if vals:
+                        values.append([child])
+                        values.append(vals)
+                    template2tax_code[child.template.id] = child.id
+            childs = sum((c.childs for c in childs), ())
+        if values:
+            cls.write(*values)
 
 
 class OpenChartTaxCodeStart(ModelView):
@@ -376,8 +397,8 @@ class TaxTemplate(ModelSQL, ModelView):
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
         super(TaxTemplate, cls).__register__(module_name)
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        cursor = Transaction().connection.cursor()
+        table = TableHandler(cls, module_name)
 
         # Migration from 1.0 group is no more required
         table.not_null_action('group', action='remove')
@@ -424,11 +445,11 @@ class TaxTemplate(ModelSQL, ModelView):
 
     @staticmethod
     def default_credit_note_base_sign():
-        return Decimal('1')
+        return Decimal('-1')
 
     @staticmethod
     def default_credit_note_tax_sign():
-        return Decimal('1')
+        return Decimal('-1')
 
     @staticmethod
     def default_update_unit_price():
@@ -456,8 +477,9 @@ class TaxTemplate(ModelSQL, ModelView):
             res['template'] = self.id
         return res
 
-    def create_tax(self, company_id, template2tax_code, template2account,
-            template2tax=None, parent_id=None):
+    @classmethod
+    def create_tax(cls, account_id, company_id, template2tax_code,
+            template2account, template2tax=None):
         '''
         Create recursively taxes based on template.
 
@@ -469,7 +491,6 @@ class TaxTemplate(ModelSQL, ModelView):
         template2tax is a dictionary with tax template id as key and tax id as
         value, used to convert template id into tax.  The dictionary is filled
         with new taxes.
-        Return id of the tax created
         '''
         pool = Pool()
         Tax = pool.get('account.tax')
@@ -477,52 +498,63 @@ class TaxTemplate(ModelSQL, ModelView):
         if template2tax is None:
             template2tax = {}
 
-        if self.id not in template2tax:
-            vals = self._get_tax_value()
-            vals['company'] = company_id
-            vals['parent'] = parent_id
-            if self.invoice_account:
-                vals['invoice_account'] = \
-                    template2account[self.invoice_account.id]
-            else:
-                vals['invoice_account'] = None
-            if self.credit_note_account:
-                vals['credit_note_account'] = \
-                    template2account[self.credit_note_account.id]
-            else:
-                vals['credit_note_account'] = None
-            if self.invoice_base_code:
-                vals['invoice_base_code'] = \
-                    template2tax_code[self.invoice_base_code.id]
-            else:
-                vals['invoice_base_code'] = None
-            if self.invoice_tax_code:
-                vals['invoice_tax_code'] = \
-                    template2tax_code[self.invoice_tax_code.id]
-            else:
-                vals['invoice_tax_code'] = None
-            if self.credit_note_base_code:
-                vals['credit_note_base_code'] = \
-                    template2tax_code[self.credit_note_base_code.id]
-            else:
-                vals['credit_note_base_code'] = None
-            if self.credit_note_tax_code:
-                vals['credit_note_tax_code'] = \
-                    template2tax_code[self.credit_note_tax_code.id]
-            else:
-                vals['credit_note_tax_code'] = None
+        def create(templates):
+            values = []
+            created = []
+            for template in templates:
+                if template.id not in template2tax:
+                    vals = template._get_tax_value()
+                    vals['company'] = company_id
+                    if template.parent:
+                        vals['parent'] = template2tax[template.parent.id]
+                    else:
+                        vals['parent'] = None
+                    if template.invoice_account:
+                        vals['invoice_account'] = \
+                            template2account[template.invoice_account.id]
+                    else:
+                        vals['invoice_account'] = None
+                    if template.credit_note_account:
+                        vals['credit_note_account'] = \
+                            template2account[template.credit_note_account.id]
+                    else:
+                        vals['credit_note_account'] = None
+                    if template.invoice_base_code:
+                        vals['invoice_base_code'] = \
+                            template2tax_code[template.invoice_base_code.id]
+                    else:
+                        vals['invoice_base_code'] = None
+                    if template.invoice_tax_code:
+                        vals['invoice_tax_code'] = \
+                            template2tax_code[template.invoice_tax_code.id]
+                    else:
+                        vals['invoice_tax_code'] = None
+                    if template.credit_note_base_code:
+                        vals['credit_note_base_code'] = \
+                            template2tax_code[
+                                template.credit_note_base_code.id]
+                    else:
+                        vals['credit_note_base_code'] = None
+                    if template.credit_note_tax_code:
+                        vals['credit_note_tax_code'] = \
+                            template2tax_code[template.credit_note_tax_code.id]
+                    else:
+                        vals['credit_note_tax_code'] = None
 
-            new_tax, = Tax.create([vals])
+                    values.append(vals)
+                    created.append(template)
 
-            template2tax[self.id] = new_tax.id
-        new_id = template2tax[self.id]
+            taxes = Tax.create(values)
+            for template, tax in zip(created, taxes):
+                template2tax[template.id] = tax.id
 
-        new_childs = []
-        for child in self.childs:
-            new_childs.append(child.create_tax(company_id, template2tax_code,
-                    template2account, template2tax=template2tax,
-                    parent_id=new_id))
-        return new_id
+        childs = cls.search([
+                ('account', '=', account_id),
+                ('parent', '=', None),
+                ])
+        while childs:
+            create(childs)
+            childs = sum((c.childs for c in childs), ())
 
 
 class Tax(ModelSQL, ModelView):
@@ -674,8 +706,8 @@ class Tax(ModelSQL, ModelView):
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
         super(Tax, cls).__register__(module_name)
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        cursor = Transaction().connection.cursor()
+        table = TableHandler(cls, module_name)
 
         # Migration from 1.0 group is no more required
         table.not_null_action('group', action='remove')
@@ -726,11 +758,11 @@ class Tax(ModelSQL, ModelView):
 
     @staticmethod
     def default_credit_note_base_sign():
-        return Decimal('1')
+        return Decimal('-1')
 
     @staticmethod
     def default_credit_note_tax_sign():
-        return Decimal('1')
+        return Decimal('-1')
 
     @staticmethod
     def default_update_unit_price():
@@ -746,6 +778,15 @@ class Tax(ModelSQL, ModelView):
             return self.company.currency.digits
         return 2
 
+    @classmethod
+    def copy(cls, taxes, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('template')
+        return super(Tax, cls).copy(taxes, default=default)
+
     def _process_tax(self, price_unit):
         if self.type == 'percentage':
             amount = price_unit * self.rate
@@ -871,7 +912,8 @@ class Tax(ModelSQL, ModelView):
         taxes = cls.sort_taxes(taxes)
         return cls._reverse_unit_compute(price_unit, taxes, date)
 
-    def update_tax(self, template2tax_code, template2account,
+    @classmethod
+    def update_tax(cls, company_id, template2tax_code, template2account,
             template2tax=None):
         '''
         Update recursively taxes based on template.
@@ -884,81 +926,88 @@ class Tax(ModelSQL, ModelView):
         value, used to convert template id into tax.  The dictionary is filled
         with new taxes.
         '''
-
         if template2tax is None:
             template2tax = {}
 
-        if self.template:
-            vals = self.template._get_tax_value(tax=self)
-            invoice_account_id = (self.invoice_account.id
-                if self.invoice_account else None)
-            if (self.template.invoice_account and
-                    invoice_account_id != template2account.get(
-                            self.template.invoice_account.id)):
-                vals['invoice_account'] = template2account.get(
-                    self.template.invoice_account.id)
-            elif (not self.template.invoice_account
-                    and self.invoice_account):
-                vals['invoice_account'] = None
-            credit_note_account_id = (self.credit_note_account.id
-                if self.credit_note_account else None)
-            if (self.template.credit_note_account and
-                    credit_note_account_id != template2account.get(
-                        self.template.credit_note_account.id)):
-                vals['credit_note_account'] = template2account.get(
-                    self.template.credit_note_account.id)
-            elif (not self.template.credit_note_account
-                    and self.credit_note_account):
-                vals['credit_note_account'] = None
-            invoice_base_code_id = (self.invoice_base_code.id
-                if self.invoice_base_code else None)
-            if (self.template.invoice_base_code and
-                    invoice_base_code_id != template2tax_code.get(
-                            self.template.invoice_base_code.id)):
-                vals['invoice_base_code'] = template2tax_code.get(
-                    self.template.invoice_base_code.id)
-            elif (not self.template.invoice_base_code
-                    and self.invoice_base_code):
-                vals['invoice_base_code'] = None
-            invoice_tax_code_id = (self.invoice_tax_code.id
-                if self.invoice_tax_code else None)
-            if (self.template.invoice_tax_code
-                    and invoice_tax_code_id != template2tax_code.get(
-                        self.template.invoice_tax_code.id)):
-                vals['invoice_tax_code'] = template2tax_code.get(
-                    self.template.invoice_tax_code.id)
-            elif (not self.template.invoice_tax_code
-                    and self.invoice_tax_code):
-                vals['invoice_tax_code'] = None
-            credit_note_base_code_id = (self.credit_note_base_code.id
-                if self.credit_note_base_code else None)
-            if (self.template.credit_note_base_code
-                    and credit_note_base_code_id != template2tax_code.get(
-                        self.template.credit_note_base_code.id)):
-                vals['credit_note_base_code'] = template2tax_code.get(
-                    self.template.credit_note_base_code.id)
-            elif (not self.template.credit_note_base_code
-                    and self.credit_note_base_code):
-                vals['credit_note_base_code'] = None
-            credit_note_tax_code_id = (self.credit_note_tax_code.id
-                if self.credit_note_tax_code else None)
-            if (self.template.credit_note_tax_code
-                    and credit_note_tax_code_id != template2tax_code.get(
-                        self.template.credit_note_tax_code.id)):
-                vals['credit_note_tax_code'] = template2tax_code.get(
-                    self.template.credit_note_tax_code.id)
-            elif (not self.template.credit_note_tax_code
-                    and self.credit_note_tax_code):
-                vals['credit_note_tax_code'] = None
+        values = []
+        childs = cls.search([
+                ('company', '=', company_id),
+                ('parent', '=', None),
+                ])
+        while childs:
+            for child in childs:
+                if child.template:
+                    vals = child.template._get_tax_value(tax=child)
+                    invoice_account_id = (child.invoice_account.id
+                        if child.invoice_account else None)
+                    if (child.template.invoice_account and
+                            invoice_account_id != template2account.get(
+                                    child.template.invoice_account.id)):
+                        vals['invoice_account'] = template2account.get(
+                            child.template.invoice_account.id)
+                    elif (not child.template.invoice_account
+                            and child.invoice_account):
+                        vals['invoice_account'] = None
+                    credit_note_account_id = (child.credit_note_account.id
+                        if child.credit_note_account else None)
+                    if (child.template.credit_note_account and
+                            credit_note_account_id != template2account.get(
+                                child.template.credit_note_account.id)):
+                        vals['credit_note_account'] = template2account.get(
+                            child.template.credit_note_account.id)
+                    elif (not child.template.credit_note_account
+                            and child.credit_note_account):
+                        vals['credit_note_account'] = None
+                    invoice_base_code_id = (child.invoice_base_code.id
+                        if child.invoice_base_code else None)
+                    if (child.template.invoice_base_code and
+                            invoice_base_code_id != template2tax_code.get(
+                                    child.template.invoice_base_code.id)):
+                        vals['invoice_base_code'] = template2tax_code.get(
+                            child.template.invoice_base_code.id)
+                    elif (not child.template.invoice_base_code
+                            and child.invoice_base_code):
+                        vals['invoice_base_code'] = None
+                    invoice_tax_code_id = (child.invoice_tax_code.id
+                        if child.invoice_tax_code else None)
+                    if (child.template.invoice_tax_code
+                            and invoice_tax_code_id != template2tax_code.get(
+                                child.template.invoice_tax_code.id)):
+                        vals['invoice_tax_code'] = template2tax_code.get(
+                            child.template.invoice_tax_code.id)
+                    elif (not child.template.invoice_tax_code
+                            and child.invoice_tax_code):
+                        vals['invoice_tax_code'] = None
+                    credit_note_base_code_id = (child.credit_note_base_code.id
+                        if child.credit_note_base_code else None)
+                    if (child.template.credit_note_base_code
+                            and (credit_note_base_code_id
+                                != template2tax_code.get(
+                                    child.template.credit_note_base_code.id))):
+                        vals['credit_note_base_code'] = template2tax_code.get(
+                            child.template.credit_note_base_code.id)
+                    elif (not child.template.credit_note_base_code
+                            and child.credit_note_base_code):
+                        vals['credit_note_base_code'] = None
+                    credit_note_tax_code_id = (child.credit_note_tax_code.id
+                        if child.credit_note_tax_code else None)
+                    if (child.template.credit_note_tax_code
+                            and (credit_note_tax_code_id
+                                != template2tax_code.get(
+                                    child.template.credit_note_tax_code.id))):
+                        vals['credit_note_tax_code'] = template2tax_code.get(
+                            child.template.credit_note_tax_code.id)
+                    elif (not child.template.credit_note_tax_code
+                            and child.credit_note_tax_code):
+                        vals['credit_note_tax_code'] = None
 
-            if vals:
-                self.write([self], vals)
-
-            template2tax[self.template.id] = self.id
-
-        for child in self.childs:
-            child.update_tax(template2tax_code, template2account,
-                template2tax=template2tax)
+                    if vals:
+                        values.append([child])
+                        values.append(vals)
+                    template2tax[child.template.id] = child.id
+            childs = sum((c.childs for c in childs), ())
+        if values:
+            cls.write(*values)
 
 
 class _TaxKey(dict):
@@ -993,11 +1042,6 @@ class TaxableMixin(object):
         """
         return []
 
-    @property
-    def tax_type(self):
-        "The taxation type it can be 'invoice' or 'credit_note'"
-        return None
-
     @property
     def currency(self):
         "The currency used by the taxable object"
@@ -1014,8 +1058,11 @@ class TaxableMixin(object):
         return {}
 
     @staticmethod
-    def _compute_tax_line(type_, amount, base, tax):
-        assert type_ in ('invoice', 'credit_note')
+    def _compute_tax_line(amount, base, tax):
+        if base >= 0:
+            type_ = 'invoice'
+        else:
+            type_ = 'credit_note'
 
         line = {}
         line['manual'] = False
@@ -1038,8 +1085,7 @@ class TaxableMixin(object):
         if not self.currency:
             return
         for taxline in taxes.itervalues():
-            for attribute in ('base', 'amount'):
-                taxline[attribute] = self.currency.round(taxline[attribute])
+            taxline['amount'] = self.currency.round(taxline['amount'])
 
     def _get_taxes(self):
         pool = Pool()
@@ -1055,7 +1101,9 @@ class TaxableMixin(object):
                 l_taxes = Tax.compute(line.taxes, line.unit_price,
                     line.quantity, self.tax_date)
                 for tax in l_taxes:
-                    taxline = self._compute_tax_line(self.tax_type, **tax)
+                    taxline = self._compute_tax_line(**tax)
+                    if self.currency:
+                        taxline['base'] = self.currency.round(taxline['base'])
                     if taxline not in taxes:
                         taxes[taxline] = taxline
                     else:
@@ -1135,13 +1183,13 @@ class TaxRuleTemplate(ModelSQL, ModelView):
             res['template'] = self.id
         return res
 
-    def create_rule(self, company_id, template2rule=None):
+    @classmethod
+    def create_rule(cls, account_id, company_id, template2rule=None):
         '''
         Create tax rule based on template.
         template2rule is a dictionary with tax rule template id as key and tax
         rule id as value, used to convert template id into tax rule. The
         dictionary is filled with new tax rules.
-        Return id of the tax rule created
         '''
         pool = Pool()
         Rule = pool.get('account.tax.rule')
@@ -1149,13 +1197,22 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         if template2rule is None:
             template2rule = {}
 
-        if self.id not in template2rule:
-            vals = self._get_tax_rule_value()
-            vals['company'] = company_id
-            new_rule, = Rule.create([vals])
+        templates = cls.search([
+                ('account', '=', account_id),
+                ])
 
-            template2rule[self.id] = new_rule.id
-        return template2rule[self.id]
+        values = []
+        created = []
+        for template in templates:
+            if template.id not in template2rule:
+                vals = template._get_tax_rule_value()
+                vals['company'] = company_id
+                values.append(vals)
+                created.append(template)
+
+        rules = Rule.create(values)
+        for template, rule in zip(created, rules):
+            template2rule[template.id] = rule.id
 
 
 class TaxRule(ModelSQL, ModelView):
@@ -1179,6 +1236,15 @@ class TaxRule(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @classmethod
+    def copy(cls, rules, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('template')
+        return super(TaxRule, cls).copy(rules, default=default)
+
     def apply(self, tax, pattern):
         '''
         Apply rule on tax
@@ -1195,23 +1261,30 @@ class TaxRule(ModelSQL, ModelView):
                 return line.get_taxes()
         return tax and [tax.id] or None
 
-    def update_rule(self, template2rule=None):
+    @classmethod
+    def update_rule(cls, company_id, template2rule=None):
         '''
         Update tax rule based on template.
         template2rule is a dictionary with tax rule template id as key and tax
         rule id as value, used to convert template id into tax rule. The
         dictionary is filled with new tax rules.
         '''
-
         if template2rule is None:
             template2rule = {}
 
-        if self.template:
-            vals = self.template._get_tax_rule_value(rule=self)
-            if vals:
-                self.write([self], vals)
-
-            template2rule[self.template.id] = self.id
+        values = []
+        rules = cls.search([
+                ('company', '=', company_id),
+                ])
+        for rule in rules:
+            if rule.template:
+                vals = rule.template._get_tax_rule_value(rule=rule)
+                if vals:
+                    values.append([rule])
+                    values.append(vals)
+            template2rule[rule.template.id] = rule.id
+        if values:
+            cls.write(*values)
 
 
 class TaxRuleLineTemplate(ModelSQL, ModelView):
@@ -1223,6 +1296,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         ondelete='RESTRICT')
     origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
         domain=[
+            ('parent', '=', None),
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
             ('group', '=', Eval('group')),
             ['OR',
@@ -1241,6 +1315,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         ondelete='RESTRICT')
     tax = fields.Many2One('account.tax.template', 'Substitution Tax',
         domain=[
+            ('parent', '=', None),
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
             ('group', '=', Eval('group')),
             ['OR',
@@ -1266,8 +1341,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
 
         super(TaxRuleLineTemplate, cls).__register__(module_name)
 
@@ -1292,7 +1366,8 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
             res['template'] = self.id
         return res
 
-    def create_rule_line(self, template2tax, template2rule,
+    @classmethod
+    def create_rule_line(cls, account_id, template2tax, template2rule,
             template2rule_line=None):
         '''
         Create tax rule line based on template.
@@ -1303,27 +1378,36 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         template2rule_line is a dictionary with tax rule line template id as
         key and tax rule line id as value, used to convert template id into tax
         rule line. The dictionary is filled with new tax rule lines.
-        Return id of the tax rule line created
         '''
         RuleLine = Pool().get('account.tax.rule.line')
 
         if template2rule_line is None:
             template2rule_line = {}
 
-        if self.id not in template2rule_line:
-            vals = self._get_tax_rule_line_value()
-            vals['rule'] = template2rule[self.rule.id]
-            if self.origin_tax:
-                vals['origin_tax'] = template2tax[self.origin_tax.id]
-            else:
-                vals['origin_tax'] = None
-            if self.tax:
-                vals['tax'] = template2tax[self.tax.id]
-            else:
-                vals['tax'] = None
-            new_rule_line, = RuleLine.create([vals])
-            template2rule_line[self.id] = new_rule_line.id
-        return template2rule_line[self.id]
+        templates = cls.search([
+                ('rule.account', '=', account_id),
+                ])
+
+        values = []
+        created = []
+        for template in templates:
+            if template.id not in template2rule_line:
+                vals = template._get_tax_rule_line_value()
+                vals['rule'] = template2rule[template.rule.id]
+                if template.origin_tax:
+                    vals['origin_tax'] = template2tax[template.origin_tax.id]
+                else:
+                    vals['origin_tax'] = None
+                if template.tax:
+                    vals['tax'] = template2tax[template.tax.id]
+                else:
+                    vals['tax'] = None
+                values.append(vals)
+                created.append(template)
+
+        rule_lines = RuleLine.create(values)
+        for template, rule_line in zip(created, rule_lines):
+            template2rule_line[template.id] = rule_line.id
 
 
 class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
@@ -1336,6 +1420,7 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
         ondelete='RESTRICT')
     origin_tax = fields.Many2One('account.tax', 'Original Tax',
         domain=[
+            ('parent', '=', None),
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
             ['OR',
@@ -1354,6 +1439,7 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
         ondelete='RESTRICT')
     tax = fields.Many2One('account.tax', 'Substitution Tax',
         domain=[
+            ('parent', '=', None),
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
             ['OR',
@@ -1380,8 +1466,7 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
     @classmethod
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
+        table = TableHandler(cls, module_name)
 
         super(TaxRuleLine, cls).__register__(module_name)
 
@@ -1393,6 +1478,15 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
         table, _ = tables[None]
         return [Case((table.sequence == Null, 0), else_=1), table.sequence]
 
+    @classmethod
+    def copy(cls, lines, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('template')
+        return super(TaxRuleLine, cls).copy(lines, default=default)
+
     def match(self, pattern):
         if 'group' in pattern and not self.group:
             if pattern['group']:
@@ -1407,7 +1501,8 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
             return [self.tax.id]
         return None
 
-    def update_rule_line(self, template2tax, template2rule,
+    @classmethod
+    def update_rule_line(cls, company_id, template2tax, template2rule,
             template2rule_line=None):
         '''
         Update tax rule line based on template.
@@ -1422,31 +1517,40 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
         if template2rule_line is None:
             template2rule_line = {}
 
-        if self.template:
-            vals = self.template._get_tax_rule_line_value(rule_line=self)
-            if self.rule.id != template2rule[self.template.rule.id]:
-                vals['rule'] = template2rule[self.template.rule.id]
-            if self.origin_tax:
-                if self.template.origin_tax:
-                    if (self.origin_tax.id !=
-                            template2tax[self.template.origin_tax.id]):
-                        vals['origin_tax'] = template2tax[
-                            self.template.origin_tax.id]
-                else:
-                    vals['origin_tax'] = None
-            elif self.template.origin_tax:
-                vals['origin_tax'] = template2tax[self.template.origin_tax.id]
-            if self.tax:
-                if self.template.tax:
-                    if self.tax.id != template2tax[self.template.tax.id]:
-                        vals['tax'] = template2tax[self.template.tax.id]
-                else:
-                    vals['tax'] = None
-            elif self.template.tax:
-                vals['tax'] = template2tax[self.template.tax.id]
-            if vals:
-                self.write([self], vals)
-            template2rule_line[self.template.id] = self.id
+        values = []
+        lines = cls.search([
+                ('rule.company', '=', company_id),
+                ])
+        for line in lines:
+            if line.template:
+                vals = line.template._get_tax_rule_line_value(rule_line=line)
+                if line.rule.id != template2rule[line.template.rule.id]:
+                    vals['rule'] = template2rule[line.template.rule.id]
+                if line.origin_tax:
+                    if line.template.origin_tax:
+                        if (line.origin_tax.id !=
+                                template2tax[line.template.origin_tax.id]):
+                            vals['origin_tax'] = template2tax[
+                                line.template.origin_tax.id]
+                    else:
+                        vals['origin_tax'] = None
+                elif line.template.origin_tax:
+                    vals['origin_tax'] = template2tax[
+                        line.template.origin_tax.id]
+                if line.tax:
+                    if line.template.tax:
+                        if line.tax.id != template2tax[line.template.tax.id]:
+                            vals['tax'] = template2tax[line.template.tax.id]
+                    else:
+                        vals['tax'] = None
+                elif line.template.tax:
+                    vals['tax'] = template2tax[line.template.tax.id]
+                if vals:
+                    values.append([line])
+                    values.append(vals)
+                template2rule_line[line.template.id] = line.id
+        if values:
+            cls.write(*values)
 
 
 class OpenTaxCode(Wizard):
@@ -1501,6 +1605,7 @@ class AccountTemplateTaxTemplate(ModelSQL):
 
 
 class AccountTemplate2:
+    __metaclass__ = PoolMeta
     __name__ = 'account.account.template'
     taxes = fields.Many2Many('account.account.template-account.tax.template',
             'account', 'tax', 'Default Taxes',
@@ -1518,6 +1623,7 @@ class AccountTax(ModelSQL):
 
 
 class Account2:
+    __metaclass__ = PoolMeta
     __name__ = 'account.account'
     taxes = fields.Many2Many('account.account-account.tax',
             'account', 'tax', 'Default Taxes',
